"use strict";
/*
 * Deepkit Framework
 * Copyright (C) 2020 Deepkit UG
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var ValidationError_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntitySubject = exports.StreamBehaviorSubject = exports.getUnserializedError = exports.getSerializedErrorPair = exports.ValidationParameterError = exports.ValidationError = exports.ValidationErrorItem = exports.JSONError = void 0;
const rxjs_1 = require("rxjs");
const core_rxjs_1 = require("@deepkit/core-rxjs");
const core_1 = require("@deepkit/core");
const buffer_1 = require("buffer");
const type_1 = require("@deepkit/type");
const operators_1 = require("rxjs/operators");
let JSONError = class JSONError {
    constructor(json) {
        this.json = json;
    }
};
JSONError = __decorate([
    type_1.Entity('@error:json'),
    __param(0, type_1.t.any.name('json')),
    __metadata("design:paramtypes", [Object])
], JSONError);
exports.JSONError = JSONError;
let ValidationErrorItem = class ValidationErrorItem {
    constructor(path, message, code) {
        this.path = path;
        this.message = message;
        this.code = code;
    }
};
ValidationErrorItem = __decorate([
    __param(0, type_1.t.name('path')),
    __param(1, type_1.t.name('message')),
    __param(2, type_1.t.name('code')),
    __metadata("design:paramtypes", [String, String, String])
], ValidationErrorItem);
exports.ValidationErrorItem = ValidationErrorItem;
let ValidationError = ValidationError_1 = class ValidationError extends core_1.CustomError {
    constructor(errors) {
        super('Validation error');
        this.errors = errors;
    }
    static from(errors) {
        return new ValidationError_1(errors.map(v => new ValidationErrorItem(v.path, v.message, v.code || '')));
    }
    get message() {
        return this.errors.map(v => `${v.path}: ${v.message} (${v.code})`).join(',');
    }
};
ValidationError = ValidationError_1 = __decorate([
    type_1.Entity('@error:validation'),
    __param(0, type_1.t.array(ValidationErrorItem).name('errors')),
    __metadata("design:paramtypes", [Array])
], ValidationError);
exports.ValidationError = ValidationError;
let ValidationParameterError = class ValidationParameterError {
    constructor(controller, action, arg, errors) {
        this.controller = controller;
        this.action = action;
        this.arg = arg;
        this.errors = errors;
    }
    get message() {
        return this.errors.map(v => `${v.path}: ${v.message} (${v.code})`).join(',');
    }
};
ValidationParameterError = __decorate([
    type_1.Entity('@error:parameter'),
    __param(0, type_1.t.name('controller')),
    __param(1, type_1.t.name('action')),
    __param(2, type_1.t.name('arg')),
    __param(3, type_1.t.array(ValidationErrorItem).name('errors')),
    __metadata("design:paramtypes", [String, String, Number, Array])
], ValidationParameterError);
exports.ValidationParameterError = ValidationParameterError;
function getSerializedErrorPair(error) {
    if (error instanceof Error) {
        return ['@error:default', error.message, error.stack];
    }
    else if ('string' === typeof error) {
        return ['@error:default', error, (new Error).stack];
    }
    else if ('function' === typeof error['constructor']) {
        const entityName = type_1.getClassSchema(error['constructor']).name;
        if (entityName) {
            return [entityName, type_1.jsonSerializer.for(error['constructor']).serialize(error), error ? error.stack : undefined];
        }
    }
    return ['@error:default', error, undefined];
}
exports.getSerializedErrorPair = getSerializedErrorPair;
function getUnserializedError(entityName, error, stack, info = '') {
    if (!entityName || entityName === '@error:default') {
        const errorObject = new Error(error);
        if (stack) {
            // console.log('error stack', errorObject.stack);
            // console.log('server stack', stack);
            errorObject.stack = errorObject.stack + `\n    at ORIGIN (${info})\n` + stack.substr(stack.indexOf('\n    at'));
            // console.log('result', errorObject.stack);
        }
        return errorObject;
    }
    if (!type_1.hasClassSchemaByName(entityName)) {
        throw new Error(`deepkit/type entity ${entityName} not known. (known: ${type_1.getKnownClassSchemasNames().join(',')})`);
    }
    return type_1.jsonSerializer.for(type_1.getClassSchemaByName(entityName).classType).deserialize(error);
}
exports.getUnserializedError = getUnserializedError;
class StreamBehaviorSubject extends rxjs_1.BehaviorSubject {
    constructor(item, teardown) {
        super(item);
        this.appendSubject = new rxjs_1.Subject();
        this.nextOnAppend = false;
        this.unsubscribed = false;
        this.teardowns = [];
        if (teardown) {
            this.teardowns.push(teardown);
        }
    }
    isUnsubscribed() {
        return this.unsubscribed;
    }
    get nextStateChange() {
        if (!this.nextChange) {
            this.nextChange = new rxjs_1.Subject();
        }
        return this.nextChange.toPromise();
    }
    addTearDown(teardown) {
        if (this.unsubscribed) {
            core_rxjs_1.tearDown(teardown);
            return;
        }
        this.teardowns.push(teardown);
    }
    /**
     * This method differs to BehaviorSubject in the way that this does not throw an error
     * when the subject is closed/unsubscribed.
     */
    getValue() {
        if (this.hasError) {
            throw this.thrownError;
        }
        else {
            return this._value;
        }
    }
    next(value) {
        super.next(value);
        if (this.nextChange) {
            this.nextChange.complete();
            delete this.nextChange;
        }
    }
    activateNextOnAppend() {
        this.nextOnAppend = true;
    }
    toUTF8() {
        const subject = new StreamBehaviorSubject(this.value instanceof Uint8Array ? type_1.arrayBufferTo(this.value, 'utf8') : '');
        const sub1 = this.pipe(operators_1.skip(1)).subscribe(v => {
            subject.next(v instanceof Uint8Array ? type_1.arrayBufferTo(v, 'utf8') : '');
        });
        const sub2 = this.appendSubject.subscribe(v => {
            subject.append(v instanceof Uint8Array ? type_1.arrayBufferTo(v, 'utf8') : '');
        });
        subject.nextOnAppend = this.nextOnAppend;
        // const that = this;
        // Object.defineProperty(subject, 'nextStateChange', {
        //     get() {
        //         console.log('utf8 nextStateChange');
        //         return that.nextStateChange;
        //     }
        // });
        subject.addTearDown(() => {
            sub1.unsubscribe();
            sub2.unsubscribe();
            this.unsubscribe();
        });
        return subject;
    }
    append(value) {
        this.appendSubject.next(value);
        if (this.nextOnAppend) {
            if (value instanceof Uint8Array) {
                if (this.value instanceof Uint8Array) {
                    this.next(buffer_1.Buffer.concat([this.value, value]));
                }
                else {
                    this.next(value);
                }
            }
            else {
                this.next((this.getValue() + value));
            }
        }
        else {
            if ('string' === typeof value) {
                if (!this._value)
                    this._value = '';
                this._value = this._value + value;
            }
        }
    }
    async unsubscribe() {
        if (this.unsubscribed)
            return;
        this.unsubscribed = true;
        for (const teardown of this.teardowns) {
            await core_rxjs_1.tearDown(teardown);
        }
        await super.unsubscribe();
    }
}
exports.StreamBehaviorSubject = StreamBehaviorSubject;
class EntitySubject extends StreamBehaviorSubject {
    constructor() {
        super(...arguments);
        /**
         * Patches are in class format.
         */
        this.patches = new rxjs_1.Subject();
        this.delete = new rxjs_1.Subject();
        this.deleted = false;
    }
    get id() {
        return this.value.id;
    }
    get onDeletion() {
        return new rxjs_1.Observable((observer) => {
            if (this.deleted) {
                observer.next();
                return;
            }
            const sub = this.delete.subscribe(() => {
                observer.next();
                sub.unsubscribe();
            });
            return {
                unsubscribe() {
                    sub.unsubscribe();
                }
            };
        });
    }
    next(value) {
        if (value === undefined) {
            this.deleted = true;
            this.delete.next(true);
            super.next(this.value);
            return;
        }
        super.next(value);
    }
}
exports.EntitySubject = EntitySubject;
//# sourceMappingURL=core.js.map