import { IdInterface } from "../index";
export declare class JobAssignedResourcesGpu {
    uuid: string;
    name: string;
    memory: number;
    constructor(uuid: string, name: string, memory: number);
}
export declare class JobAssignedResources {
    cpu: number;
    memory: number;
    gpus: JobAssignedResourcesGpu[];
    getMinGpuMemory(): number;
    getMaxGpuMemory(): number;
    getGpuUUIDs(): string[];
    getGpuMemoryRange(): string;
}
export declare class JobConfigDocker {
    env: string[];
    binds: string[];
    links: string[];
}
export declare class JobResources {
    minCpu: number;
    maxCpu: number;
    cpu: number;
    allMemory: boolean;
    minMemory: number;
    maxMemory: number;
    memory: number;
    minGpu: number;
    maxGpu: number;
    gpu: number;
    /**
     * Value in GB. Defines minimum gpu memory that is necessary.
     */
    minGpuMemory: number;
    static fromPartial(partial: Partial<JobResources>): JobResources;
    normalizeValues(): void;
    getMinCpu(): number;
    getMaxCpu(): number;
    getMinMemory(): number;
    getMaxMemory(): number;
    getMinGpu(): number;
    getMaxGpu(): number;
}
export declare class JobTaskCommand {
    name: string;
    command: string;
    constructor(name?: string, command?: string);
}
export declare class JobTaskConfigBase {
    label: string;
    install: string[];
    dockerfile: string;
    install_files: string[];
    image: string;
    output: string[];
    environmentVariables: string[];
    nodeIds: string[];
    nodes: string[];
    clusters: string[];
    commands: JobTaskCommand[];
    args: string[];
    resources: JobResources;
    docker: JobConfigDocker;
    isDockerImage(): boolean;
    hasCommand(): boolean;
    get installString(): string;
    set installString(v: string);
    set installFilesString(v: string);
    get installFilesString(): string;
}
export declare class JobTaskConfig extends JobTaskConfigBase {
    /**
     * Will be set by config loader.
     */
    name: string;
    replicas: number;
    depends_on: string[];
    /**
     * Will be set config loader
     */
    configValuesNoOverwrite: {
        [path: string]: true;
    };
    isRoot(): boolean;
}
export declare class JobConfig extends JobTaskConfigBase {
    static readonly inheritTaskProperties: string[];
    parameters: {
        [name: string]: any;
    };
    path: string;
    ignore: string[];
    priority: number;
    import: string;
    protected tasks: {
        [name: string]: JobTaskConfig;
    };
    protected parametersArray?: {
        name: string;
        value: any;
    }[];
    protected resolved: boolean;
    getLabel(): string;
    /**
     * Writes all values of the root config into task config and the values from the original config (configValuesToOverwrite)
     * file to task config (so user can override them)
     */
    resolveInheritance(): void;
    getTasks(): {
        [name: string]: JobTaskConfig;
    };
}
export declare class JobEnvironmentPython {
    version?: string;
    binary?: string;
    sdkVersion?: string;
    pipPackages: {
        [name: string]: string;
    };
}
export declare class JobEnvironment {
    hostname?: string;
    username?: string;
    platform?: string;
    release?: string;
    arch?: string;
    uptime?: number;
    nodeVersion?: string;
    environmentVariables?: {
        [name: string]: string;
    };
    python?: JobEnvironmentPython;
}
export declare class JobGit {
    commit: string;
    message: string;
    authorEmail?: string | undefined;
    date?: Date | undefined;
    author?: string;
    branch?: string;
    origin?: string;
    constructor(commit: string, message: string, authorEmail?: string | undefined, date?: Date | undefined);
}
export declare class JobDocker {
    runOnVersion?: string;
}
export declare class JobDockerImage {
    name?: string;
    id?: string;
    size?: number;
    os?: string;
    arch?: string;
    created?: Date;
    builtWithDockerVersion?: string;
}
export declare enum JobStatus {
    creating = 0,
    created = 50,
    running = 100,
    done = 150,
    aborted = 200,
    failed = 250,
    crashed = 300
}
export declare enum JobTaskStatus {
    pending = 0,
    queued = 100,
    assigned = 150,
    started = 300,
    done = 500,
    aborted = 550,
    failed = 600,
    crashed = 650
}
export declare enum JobTaskInstanceStatus {
    pending = 0,
    booting = 200,
    docker_pull = 220,
    docker_build_await = 230,
    docker_build = 235,
    joining_network = 250,
    checkout_files = 260,
    started = 300,
    done = 500,
    aborted = 550,
    failed = 600,
    crashed = 650
}
export declare class Channel {
    /**
     * This might be empty.
     */
    traces: string[];
    kpi?: boolean;
    kpiTrace: number;
    maxOptimization: boolean;
    lastValue: any[];
    xaxis?: object;
    yaxis?: object;
    layout?: object;
}
export declare enum PullStatsStatus {
    waiting = "waiting",
    downloading = "downloading",
    extracting = "extracting",
    verifying = "verifying",
    done = "done"
}
export declare class PullStats {
    readonly id: string;
    current: number;
    total: number;
    status: PullStatsStatus;
    constructor(id: string);
}
export declare class JobTaskInstance {
    id: number;
    status: JobTaskInstanceStatus;
    uploadOutputCurrent: number;
    uploadOutputTotal: number;
    environment: JobEnvironment;
    docker: JobDocker;
    dockerPullStats: {
        [id: string]: PullStats;
    };
    dockerImage: JobDockerImage;
    node?: string;
    exitCode?: number;
    error: string;
    assignedResources: JobAssignedResources;
    started?: Date;
    ended?: Date;
    constructor(id: number);
    getOrCreatePullStats(id: string): PullStats;
    isRunning(): boolean;
    isStarted(): boolean;
    isEnded(): boolean;
    isDockerPull(): boolean;
    elapsedTime(): number | undefined;
}
export declare class JobTaskQueue {
    position: number;
    tries: number;
    result: string;
    added: Date;
}
export declare class JobTask {
    name: string;
    queue: JobTaskQueue;
    status: JobTaskStatus;
    assigned?: Date;
    started?: Date;
    ended?: Date;
    exitCode?: number;
    instances: JobTaskInstance[];
    constructor(name: string, replicas: number);
    elapsedTime(): number | undefined;
    getInstances(): JobTaskInstance[];
    getRunningInstances(): JobTaskInstance[];
    getFirstInstance(): JobTaskInstance | undefined;
    isErrored(): boolean;
    getInstance(replica: number): JobTaskInstance;
    isStarted(): boolean;
    isQueued(): boolean;
    isRunning(): boolean;
    isEnded(): boolean;
    areAllInstancesEnded(): boolean;
    calculateStatusByInstances(): JobTaskStatus;
}
export declare class JobModelGraphSnapshotHistogram {
    x: number[];
    y: number[];
}
export declare class JobModelGraphSnapshotLayer {
    saved?: Date;
    outputFileId?: string;
    weights?: JobModelGraphSnapshotHistogram;
    biases?: JobModelGraphSnapshotHistogram;
}
export declare class JobModelGraphSnapshot {
    job: string;
    id: string;
    version: number;
    created: Date;
    iteration: number;
    live: boolean;
    layerInfo: {
        [layerName: string]: JobModelGraphSnapshotLayer;
    };
    constructor(job: string);
}
export declare class JobModelNode {
    /**
     * Separated by /. Will be grouped together.
     */
    id: string;
    label: string;
    type: string;
    op?: string;
    shape: number[];
    attributes: {
        [name: string]: any;
    };
    /**
     * input ids from other nodes.
     */
    input: string[];
    /**
     * input ids from other nodes.
     */
    children: JobModelNode[];
    hasSelfActivation(): boolean;
    isSmallBox(): boolean;
    hasBorder(): boolean;
    isBorderRadius(): boolean;
    isActivation(): boolean;
}
export declare class JobModelGraph {
    nodes: JobModelNode[];
    inputs: string[];
    outputs: string[];
}
export declare class Job implements IdInterface {
    id: string;
    project: string;
    accessToken: string;
    number: number;
    version: number;
    description: string;
    /**
     * The cluster id if assigned.
     */
    cluster?: string;
    connections: number;
    created: Date;
    updated: Date;
    author?: string;
    config: JobConfig;
    git?: JobGit;
    configFile?: string;
    /**
     * Whether the job is executed directly in script without Deepkit CLI tools.
     */
    selfExecution: boolean;
    hasModelGraph: boolean;
    liveSnapshotId?: string;
    status: JobStatus;
    title: string;
    infos: {
        [name: string]: any;
    };
    tasks: {
        [name: string]: JobTask;
    };
    debugSnapshots: {
        [timestamp: number]: any;
    };
    debugActiveLayerWatching: {
        [layerId: string]: boolean;
    };
    runOnCluster: boolean;
    tags: string[];
    assigned?: Date;
    started?: Date;
    ended?: Date;
    stopRequested?: Date;
    ping?: Date;
    iteration: number;
    iterations: number;
    secondsPerIteration: number;
    step: number;
    steps: number;
    stepLabel: string;
    /**
     * ETA in seconds. Time left.
     */
    eta: number;
    speed: number;
    speedLabel: string;
    channels: {
        [name: string]: Channel;
    };
    constructor(id: string, project: string);
}
